@@ -1,7 +1,7 @@
+import io
 import warnings
 from itertools import permutations
 
-import io
 import matplotlib.image as mpimg
 import matplotlib.pyplot as plt
 import networkx as nx
@@ -41,6 +41,11 @@ class CausalGraph:
         self.labels = {}
         self.prt_m = {}  # store the parents of missingness indicators
         self.mvpc = None
+        self.cardinalities = None  # only works when self.data is discrete, i.e. self.test is chisq or gsq
+        self.is_discrete = False
+        self.citest_cache = dict()
+        self.data_hash_key = None
+        self.ci_test_hash_key = None
 
     def set_ind_test(self, indep_test, mvpc=False):
         """Set the conditional independence test that will be used"""
@@ -48,12 +53,23 @@ class CausalGraph:
         if mvpc:
             self.mvpc = True
         self.test = indep_test
+        self.ci_test_hash_key = hash(indep_test)
 
     def ci_test(self, i, j, S):
         """Define the conditional independence test"""
+        # assert i != j and not i in S and not j in S
         if self.mvpc:
             return self.test(self.data, self.nx_skel, self.prt_m, i, j, S, self.data.shape[0])
-        return self.test(self.data, i, j, S)
+
+        i, j = (i, j) if (i < j) else (j, i)
+        ijS_key = (i, j, frozenset(S), self.data_hash_key, self.ci_test_hash_key)
+        if ijS_key in self.citest_cache:
+            return self.citest_cache[ijS_key]
+        # if discrete, assert self.test is chisq or gsq, pass into cardinalities
+        pValue = self.test(self.data, i, j, S, self.cardinalities) if self.is_discrete \
+            else self.test(self.data, i, j, S)
+        self.citest_cache[ijS_key] = pValue
+        return pValue
 
     def neighbors(self, i):
         """Find the neighbors of node i in adjmat"""
@@ -61,13 +77,7 @@ class CausalGraph:
 
     def max_degree(self):
         """Return the maximum number of edges connected to a node in adjmat"""
-        nodes = range(len(self.G.graph))
-        max_degree = 0
-        for i in nodes:
-            len_neigh_i = len(self.neighbors(i))
-            if len_neigh_i > max_degree:
-                max_degree = len_neigh_i
-        return max_degree
+        return max(np.sum(self.G.graph != 0, axis=1))
 
     def find_arrow_heads(self):
         """Return the list of i o-> j in adjmat as (i, j)"""
@@ -90,8 +100,8 @@ class CausalGraph:
     def find_bi_directed(self):
         """Return the list of bidirected edges i <-> j in adjmat as (i, j) [with symmetry]"""
         return [(edge[1], edge[0]) for edge in self.find_arrow_heads() if (
-                    self.G.graph[edge[1], edge[0]] == Endpoint.ARROW.value and self.G.graph[
-                edge[0], edge[1]] == Endpoint.ARROW.value)]
+                self.G.graph[edge[1], edge[0]] == Endpoint.ARROW.value and self.G.graph[
+            edge[0], edge[1]] == Endpoint.ARROW.value)]
 
     def find_adj(self):
         """Return the list of adjacencies i --- j in adjmat as (i, j) [with symmetry]"""
@@ -155,7 +165,7 @@ class CausalGraph:
     def to_nx_graph(self):
         """Convert adjmat into a networkx.Digraph object named nx_graph"""
         nodes = range(len(self.G.graph))
-        self.labels = {i:self.G.nodes[i].get_name() for i in nodes}
+        self.labels = {i: self.G.nodes[i].get_name() for i in nodes}
         self.nx_graph.add_nodes_from(nodes)
         undirected = self.find_undirected()
         directed = self.find_fully_directed()
